/***********      .---.         .-"-.      ******************\
* -------- *     /   ._.       /  ` \     * --------------- *
* Author's *     \_  (__\      \_v_/     * humus@comhem.se *
*   note   *     //   \\       //   \\     * ICQ #47010716   *
* -------- *    ((     ))     ((     ))    * --------------- *
*          ****--""---""-------""---""--****                 *********\
* This file is a part of the work done by Humus. You are free to use  *
* the code in any way you like, modified, unmodified or copy'n'pasted *
* into your own work. However, I expect you to respect these points:  *
*  @ If you use this file and its contents unmodified, or use a major *
*    part of this file, please credit the author and leave this note. *
*  @ For use in anything commercial, please request my approval.      *
*  @ Share your work and ideas too as much as you can.                *
\*********************************************************************/

#include "App.h"

BaseApp *app = new App();

void App::resetCamera(){
	camPos = vec3(0, 0, -1);
	wx = 0;
	wy = 0;
	speed = 1.0f;
}

bool App::init(){
	depthBits = 0;
	return true;
}

void App::exit(){
}

bool App::load(){
	// Shaders
	if ((compress[0] = renderer->addShader("compress.shd")) == SHADER_NONE) return false;
	if ((compress[1] = renderer->addShader("compress.shd", "#define USE_LEAST_SQUARES\n")) == SHADER_NONE) return false;
	if ((viewTex  = renderer->addShader("viewTex.shd")) == SHADER_NONE) return false;

	// Textures
	if ((srcTex = renderer->addTexture("Image.jpg", false)) == TEXTURE_NONE) return false;

	// Render targets
	if ((dstTex = renderer->addRenderTarget(256, 256, FORMAT_RG32UI, SS_NONE)) == TEXTURE_NONE) return false;

	// Create a BC1 compressed texture. The 1024x1024 size is 256x256 blocks of 4x4 blocks,
	// with each tile being 64bits, which matches the 256x256 RG32UI render target above.
	D3D10_TEXTURE2D_DESC desc;
	desc.Width  = 1024;
	desc.Height = 1024;
	desc.Format = DXGI_FORMAT_BC1_UNORM;
	desc.MipLevels = 1;
	desc.SampleDesc.Count = 1;
	desc.SampleDesc.Quality = 0;
	desc.Usage = D3D10_USAGE_DEFAULT;
	desc.BindFlags = D3D10_BIND_SHADER_RESOURCE;
	desc.CPUAccessFlags = 0;
	desc.ArraySize = 1;
	desc.MiscFlags = 0;

	ID3D10Texture2D *tex;
	device->CreateTexture2D(&desc, NULL, &tex);

	if ((resTex = ((Direct3D10Renderer *) renderer)->addTexture(tex)) == TEXTURE_NONE) return false;

	return true;
}

void App::unload(){
}

void App::drawFrame(){
	float4x4 projection = toD3DProjection(perspectiveMatrixX(1.5f, width, height, 0.001f, 10.0f));
	float4x4 view = rotateXY(-wx, -wy) * translate(-camPos);
	float4x4 viewProj = projection * view;

	renderer->clear(true, false, false, float4(0.2f, 0.4f, 0.3f, 0));

	/*
		Compression pass
	*/
	renderer->changeRenderTarget(dstTex);
		renderer->reset();
		renderer->setRasterizerState(cullNone);
		renderer->setShader(keys[KEY_F6]? compress[0] : compress[1]);
		renderer->setTexture("Tex", srcTex);
		renderer->setDepthState(noDepthTest);
		renderer->apply();

		device->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
		device->Draw(3, 0);
	renderer->changeToMainFramebuffer();

	/*
		Copy the result to BC1 texture
	*/
	device->CopyResource(
		((Direct3D10Renderer *) renderer)->getResource(resTex),
		((Direct3D10Renderer *) renderer)->getResource(dstTex));

	/*
		Display texture
	*/
	renderer->reset();
	renderer->setRasterizerState(cullNone);
	renderer->setShader(viewTex);
	renderer->setShaderConstant4x4f("viewProj", viewProj);
	renderer->setTexture("Tex", keys[KEY_F5]? srcTex : resTex);
	renderer->setSamplerState("filter", linearClamp);
	renderer->setDepthState(noDepthTest);
	renderer->apply();

	device->IASetPrimitiveTopology(D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP);
	device->Draw(4, 0);
}
